package timing.ukulele.auth.security.type.wechat;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.util.StringUtils;
import timing.ukulele.auth.security.LoginTypeEnum;
import timing.ukulele.auth.security.ThirdTypeEnum;

public class ThirdQrAuthenticationFilter extends AbstractAuthenticationProcessingFilter {
    private boolean postOnly = true;
    private final String thirdTypeParameter = "thirdType";
    private final String codeParameter = "code";

    public ThirdQrAuthenticationFilter() {
        super(new AntPathRequestMatcher(LoginTypeEnum.THIRD_QR.getPath(), "POST"));
    }

    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) throws AuthenticationException {
        if (this.postOnly && !request.getMethod().equals("POST")) {
            throw new AuthenticationServiceException("Authentication method not supported: " + request.getMethod());
        } else {
            String code = this.obtainCode(request);
            code = code != null ? code : "";
            ThirdTypeEnum type = this.obtainThirdType(request);
            if (type == null) {
                throw new InsufficientAuthenticationException("暂不支持该平台");
            }
            if (!StringUtils.hasLength(code)) {
                throw new AuthenticationServiceException("参数错误");
            }
            ThirdQrAuthenticationToken authRequest = new ThirdQrAuthenticationToken(code);
            this.setDetails(request, authRequest);
            return this.getAuthenticationManager().authenticate(authRequest);
        }
    }

    protected ThirdTypeEnum obtainThirdType(HttpServletRequest request) {
        String parameter = request.getParameter(this.thirdTypeParameter);
        return ThirdTypeEnum.getByCode(parameter);
    }

    protected String obtainCode(HttpServletRequest request) {
        return request.getParameter(this.codeParameter);
    }

    protected void setDetails(HttpServletRequest request, ThirdQrAuthenticationToken authRequest) {
        authRequest.setDetails(this.authenticationDetailsSource.buildDetails(request));
    }

    public void setPostOnly(boolean postOnly) {
        this.postOnly = postOnly;
    }
}